package com.nsgk.ruralWeb.location;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.os.CancellationSignal;
import android.util.Log;
import android.widget.Toast;

import com.nsgk.ruralWeb.sys.NSConstants;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

public class NSLastKnownLocation
{
    private static final String ID_TAG = NSLastKnownLocation.class.getName();

    private final Context        m_context;
    private       NSLocationInfo m_lastLocation = null;

    public NSLastKnownLocation(Context context)
    {
        m_context = context;
    }

    @SuppressLint("MissingPermission")
    public NSLocationInfo Read(String type)
    {
        NSLocationInfo loc = null;
        try
        {
            LocationManager lm = (LocationManager) m_context.getSystemService(Context.LOCATION_SERVICE);

            List<String> allProviders = lm.getAllProviders();
            Log.i(ID_TAG, "使用最近所有定位提供器: " + allProviders);
            Map<String, NSLocationInfo> map = new HashMap<>();
            for(String provider : allProviders)
            {
                Location lastKnownLocation = lm.getLastKnownLocation(provider);
                if(null == lastKnownLocation)
                    continue;
                NSLocationInfo l = new NSLocationInfo(provider, lastKnownLocation.getLongitude(), lastKnownLocation.getLatitude());
                map.put(provider, l);
            }

            Log.i(ID_TAG, "最近定位提供器结果: " + map);
            if(null != type && !type.isEmpty() && map.containsKey(type))
                loc = map.get(type);
            else if(map.containsKey(LocationManager.GPS_PROVIDER))
                loc = map.get(LocationManager.GPS_PROVIDER);
            else if(map.containsKey(LocationManager.NETWORK_PROVIDER))
                loc = map.get(LocationManager.NETWORK_PROVIDER);
            else if(map.containsKey(NSConstants.FUSED_PROVIDER))
                loc = map.get(NSConstants.FUSED_PROVIDER);
            else if(map.containsKey(LocationManager.PASSIVE_PROVIDER))
                loc = map.get(LocationManager.PASSIVE_PROVIDER);
            if(null == loc)
                Log.i(ID_TAG, "所有提供器无最近定位");
            else
            {
                Log.i(ID_TAG, "使用提供器最近定位: " + loc.provider);
                m_lastLocation = loc;
            }
        }
        catch(Throwable e)
        {
            e.printStackTrace();
        }
        return loc;
    }

    public NSLocationInfo GetLastLocation()
    {
        return m_lastLocation;
    }
}