package com.nsgk.ruralWeb.ui;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.provider.Settings;
import android.text.Html;
import android.util.Log;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.core.app.NotificationCompat;
import androidx.preference.Preference;
import androidx.preference.PreferenceFragmentCompat;
import androidx.preference.PreferenceManager;
import androidx.preference.SeekBarPreference;

import com.nsgk.ruralWeb.BuildConfig;
import com.nsgk.ruralWeb.R;
import com.nsgk.ruralWeb.enums.NSEnums;
import com.nsgk.ruralWeb.sys.NSConstants;
import com.nsgk.ruralWeb.sys.NSPreference;
import com.nsgk.ruralWeb.utils.NSContextUtils;
import com.nsgk.ruralWeb.utils.NSMisc;
import com.nsgk.ruralWeb.utils.NSStr;

import java.io.File;
import java.util.Objects;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.StreamProgress;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;

public class SettingsFragment extends PreferenceFragmentCompat implements Preference.OnPreferenceChangeListener, Preference.OnPreferenceClickListener
{
    private static final String ID_TAG = SettingsFragment.class.getName();

    @Override
    public void onCreatePreferences(Bundle savedInstanceState, String rootKey) {
        addPreferencesFromResource(R.xml.settings_preference);
        Preference preference;
        final Context context = getContext();

        preference = findPreference(NSPreference.LOCATION_MODE);
        preference.setDefaultValue(NSConstants.DEFAULT_LOCATION_MODE);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_GAODE_INTERVAL);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_GAODE_READ_COUNT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.OPEN_LAST_URL);
        preference.setDefaultValue(NSConstants.DEFAULT_OPEN_LAST_URL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_PROVIDER);
        preference.setDefaultValue(NSConstants.DEFAULT_LOCATION_PROVIDER);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_INTERVAL);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_DISTANCE);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_REALTIME_READ_COUNT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.FONT_SCALE);
        preference.setDefaultValue(NSConstants.DEFAULT_FONT_SCALE);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOGCAT_CONSOLE_OUTPUT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOGCAT_CONSOLE_OUTPUT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.LOCATION_TIMEOUT);
        preference.setDefaultValue("" + NSConstants.DEFAULT_LOCATION_TIMEOUT);
        preference.setOnPreferenceChangeListener(this);

        preference = findPreference(NSPreference.RESET_SETTINGS);
        preference.setOnPreferenceClickListener(this);

        preference = findPreference(NSPreference.VERSION);
        preference.setOnPreferenceClickListener(this);

        preference = findPreference(NSPreference.UPDATE_DOWNLOAD);
        preference.setOnPreferenceClickListener(this);
        if(StrUtil.isBlank(NSConstants.AppUpdateUrl()))
            preference.setVisible(false);

        // 总是隐藏高德的设置
        if(!NSConstants.IsAMapLocationEnabled() || !NSContextUtils.BuildIsDebug(context))
        {
            preference = findPreference("location_gaode");
            preference.setVisible(false);
        }
    }

    private boolean CheckValueIsNumber(Object newValue, Integer min)
    {
        if(null != newValue)
        {
            String str = newValue.toString();
            if(StrUtil.isNotBlank(str))
            {
                if(!NumberUtil.isInteger(str))
                {
                    Toast.makeText(getContext(), "请输入数字", Toast.LENGTH_SHORT).show();
                    return false;
                }
                int i = Integer.parseInt(str);
                if(null != min)
                {
                    if(i < min)
                    {
                        Toast.makeText(getContext(), "必须大于等于" + min, Toast.LENGTH_SHORT).show();
                        return false;
                    }
                }
            }
        }
        return true;
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object newValue) {
        String key = preference.getKey();

        switch(key)
        {
            case NSPreference.LOCATION_GAODE_INTERVAL:
                if(!CheckValueIsNumber(newValue, 1000))
                    return false;
                break;
            case NSPreference.LOCATION_GAODE_READ_COUNT:
                if(!CheckValueIsNumber(newValue, 1))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_INTERVAL:
                if(!CheckValueIsNumber(newValue, 1000))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_DISTANCE:
                if(!CheckValueIsNumber(newValue, 0))
                    return false;
                break;
            case NSPreference.LOCATION_REALTIME_READ_COUNT:
                if(!CheckValueIsNumber(newValue, 1))
                    return false;
                break;
            case NSPreference.LOCATION_TIMEOUT:
                if(!CheckValueIsNumber(newValue, 0))
                    return false;
                break;
            case NSPreference.FONT_SCALE: {
                int i = (int)newValue;
                int newi = Math.round((float)i / 10.0f) * 10;
                if(i < 50)
                    i = 50;
                if(i != newi)
                {
                    getView().post(() -> {
                        ((SeekBarPreference)preference).setValue(newi);
                        preference.callChangeListener(newi);
                    });
                    return false;
                }
                break;
            }
            case NSPreference.LOCATION_MODE:
                if(!NSConstants.IsAMapLocationEnabled() && Objects.equals(NSEnums.LocationMode.GAODE, newValue))
                {
                    Log.w(ID_TAG, "高德定位被禁用");
                    return false;
                }
                if(Objects.equals(NSEnums.LocationMode.H5, newValue))
                {
                    if(!NSConstants.IsHttps())
                    {
                        Log.w(ID_TAG, "非https不支持H5定位");
                        Toast.makeText(getContext(), "当前不支持H5定位", Toast.LENGTH_SHORT).show();
                        return false;
                    }
                }
                break;
        }

        SetSummary(key, newValue);
        return true;
    }

    @Override
    public boolean onPreferenceClick(final Preference preference) {
        String key = preference.getKey();
        final Context context = getContext();

        switch(key)
        {
            case NSPreference.RESET_SETTINGS:
                OpenQueryDialog("警告", "确定要重置到默认设置?", (dialog, which) -> {
                    ResetSettings();
                    SetSummary(null, null);
                    Toast.makeText(context, "设置已重置", Toast.LENGTH_LONG).show();
                });
                break;
            case NSPreference.VERSION:
                OpenAbout();
                break;
            case NSPreference.UPDATE_DOWNLOAD:
                DownloadUpdate();
                //DownloadUpdateExternally();
                break;
        }
        return false;
    }

    private void DownloadUpdateExternally()
    {
        String updateUrl = NSConstants.AppUpdateUrl();
        if(StrUtil.isBlank(updateUrl))
            return;

        Context context = getContext();
        Toast.makeText(context, "正在打开下载地址......", Toast.LENGTH_SHORT).show();
        NSContextUtils.OpenUrlExternally(context, updateUrl);
    }

    private void DownloadUpdate()
    {
        String updateUrl = NSConstants.AppUpdateUrl();
        if(StrUtil.isBlank(updateUrl))
            return;

        Handler handler = new Handler(Looper.myLooper());
        Context context = getContext();
        Log.d(ID_TAG, "下载apk地址: " + updateUrl);
        Toast.makeText(context, "开始下载......", Toast.LENGTH_SHORT).show();

        int icon = context.getResources().getIdentifier("ic_launcher_" + NSConstants.AppIcon()/* + "_round"*/, "mipmap", context.getApplicationContext().getPackageName());
        NotificationManager notificationManager = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

        String channelId = "nsgk";
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationChannel notificationChannel = notificationManager.getNotificationChannel(channelId);
            if(null == notificationChannel)
            {
                NotificationChannel channel = new NotificationChannel(channelId, "下载更新", NotificationManager.IMPORTANCE_DEFAULT);
                channel.setDescription("下载更新");
                notificationManager.createNotificationChannel(channel);
            }
        }

        NotificationCompat.Builder builder = new NotificationCompat.Builder(context, channelId);
        builder.setProgress(0, 0, true)
                .setSmallIcon(icon)
                .setTicker("下载更新")
                .setOngoing(true)
                .setWhen(System.currentTimeMillis())
                .setContentTitle("下载更新")
                .setContentText("开始下载......")
        ;
        int notifyId = 0x1003;
        Notification notification = builder.build();
        notificationManager.notify(notifyId, notification);

        // 创建文件夹并删除缓存文件
        File externalFilesDir = context.getExternalFilesDir(null);
        String dir = externalFilesDir.getAbsolutePath() + File.separator + NSConstants.APK_UPDATE_DOWNLOAD_DIR;
        Log.d(ID_TAG, "创建下载缓存目录: " + dir);
        notificationManager.notify(notifyId, builder.setContentText("创建下载缓存目录").build());
        FileUtil.mkdir(dir);

        String file = dir + File.separator + NSConstants.APK_UPDATE_DOWNLOAD_FILE;
        Log.d(ID_TAG, "删除历史缓存文件: " + file);
        notificationManager.notify(notifyId, builder.setContentText("删除历史缓存文件").build());
        FileUtil.del(file);

        new Thread(() -> {
            long bytes = HttpUtil.downloadFile(updateUrl, new File(file), new StreamProgress() {
                @Override
                public void start()
                {
                    handler.post(() -> {
                        notificationManager.notify(notifyId, builder.setContentText("开始下载......").setProgress(100, 0, false).build());
                    });
                }

                @Override
                public void progress(long progressSize)
                {
                    handler.post(() -> {
                        notificationManager.notify(notifyId, builder.setContentText("下载: " + FileUtil.readableFileSize(progressSize)).setProgress(0, 0, true).build());
                    });
                }

                @Override
                public void finish()
                {
                    handler.post(() -> {
                        notificationManager.notify(notifyId, builder.setContentText("下载完成").setProgress(100, 100, false).setOngoing(false).build());
                    });
                }
            });
            Log.d(ID_TAG, "下载apk文件: " + FileUtil.readableFileSize(bytes));
            if(bytes > 0)
            {
                handler.postDelayed(() -> {
                    notificationManager.notify(notifyId, builder.setContentText("准备安装更新").setProgress(0, 0, false).setOngoing(false).build());
                    Toast.makeText(context, "准备安装......", Toast.LENGTH_SHORT).show();
                    NSContextUtils.InstallApk(context, file);

//                    handler.postDelayed(() -> {
//                        notificationManager.cancel(notifyId);
//                    }, 2000);
                }, 1000);
            }
            else
            {
                handler.postDelayed(() -> {
                    notificationManager.notify(notifyId, builder.setContentText("下载更新失败").setProgress(0, 0, false).setOngoing(false).build());
                    Toast.makeText(context, "下载更新失败", Toast.LENGTH_LONG).show();

//                    handler.postDelayed(() -> {
//                        notificationManager.cancel(notifyId);
//                    }, 2000);
                }, 1000);
            }
        }).start();
    }

    private void ResetSettings()
    {
        NSPreference preference = new NSPreference(getContext());
        preference.Write()
                .putString(NSPreference.LOCATION_MODE, NSConstants.DEFAULT_LOCATION_MODE)
                .putString(NSPreference.LOCATION_GAODE_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL)
                .putString(NSPreference.LOCATION_GAODE_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT)
                .putBoolean(NSPreference.OPEN_LAST_URL, NSConstants.DEFAULT_OPEN_LAST_URL)
                .putString(NSPreference.LOCATION_PROVIDER, NSConstants.DEFAULT_LOCATION_PROVIDER)
                .putString(NSPreference.LOCATION_REALTIME_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL)
                .putString(NSPreference.LOCATION_REALTIME_DISTANCE, "" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE)
                .putString(NSPreference.LOCATION_REALTIME_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT)
                .putInt(NSPreference.FONT_SCALE, NSConstants.DEFAULT_FONT_SCALE)
                .putString(NSPreference.LOCATION_TIMEOUT, "" + NSConstants.DEFAULT_LOCATION_TIMEOUT)
                .putBoolean(NSPreference.LOGCAT_CONSOLE_OUTPUT, NSConstants.DEFAULT_LOGCAT_CONSOLE_OUTPUT)
                .commit()
        ;
    }

    private void OpenQueryDialog(String title, String message, DialogInterface.OnClickListener listener)
    {
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setIcon(R.drawable.no_words);
        builder.setPositiveButton("确定", listener);
        builder.setNegativeButton("取消", null);
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    private void OpenAbout()
    {
        Context context = getContext();
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        String msg = "<br/>" + NSContextUtils.tr(context, R.string.company) + "<br/><br/>" + NSContextUtils.tr(context, R.string.copyright);
        msg = "<div style='text-align:center;'>" + msg + "</div>";
        builder.setTitle(R.string.app_name2);
        builder.setMessage(Html.fromHtml(msg, Html.FROM_HTML_MODE_LEGACY, null, null));
        int icon = context.getResources().getIdentifier("ic_launcher_" + NSConstants.AppIcon()/* + "_round"*/, "mipmap", context.getApplicationContext().getPackageName());
        builder.setIcon(icon);
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    @Override
    public void onResume() {
        super.onResume();
        SetSummary(null, null);
    }

    public void SetSummary(String key, Object newValue)
    {
        Context context = getContext();
        Preference preference;
        String summary;
        String value;
        boolean b;
        int i;
        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(context);

        if(key == null || NSPreference.LOCATION_MODE.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_MODE);
            value = newValue != null ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_MODE, NSConstants.DEFAULT_LOCATION_MODE);
            summary = NSContextUtils.GetListName(context, value, R.array.location_mode_values, R.array.location_mode_labels);
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_GAODE_INTERVAL.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_GAODE_INTERVAL);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_GAODE_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_GAODE_INTERVAL);
            summary = value + "毫秒";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_GAODE_READ_COUNT.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_GAODE_READ_COUNT);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_GAODE_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_GAODE_READ_COUNT);
            summary = value + "次";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.OPEN_LAST_URL.equals(key))
        {
            preference = findPreference(NSPreference.OPEN_LAST_URL);
            b = newValue != null ? (Boolean)newValue : sharedPreferences.getBoolean(NSPreference.OPEN_LAST_URL, NSConstants.DEFAULT_OPEN_LAST_URL);
            summary = b ? "打开最近页面" : "打开默认页面";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_PROVIDER.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_PROVIDER);
            value = newValue != null ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_PROVIDER, NSConstants.DEFAULT_LOCATION_PROVIDER);
            summary = NSContextUtils.GetListName(context, value, R.array.location_provider_values, R.array.location_provider_labels);
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_INTERVAL.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_INTERVAL);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_INTERVAL, "" + NSConstants.DEFAULT_LOCATION_REALTIME_INTERVAL);
            summary = value + "毫秒";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_DISTANCE.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_DISTANCE);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_DISTANCE, "" + NSConstants.DEFAULT_LOCATION_REALTIME_DISTANCE);
            summary = value + "米";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_REALTIME_READ_COUNT.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_REALTIME_READ_COUNT);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_REALTIME_READ_COUNT, "" + NSConstants.DEFAULT_LOCATION_REALTIME_READ_COUNT);
            summary = value + "次";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.FONT_SCALE.equals(key))
        {
            preference = findPreference(NSPreference.FONT_SCALE);
            i = newValue != null ? (int)newValue : sharedPreferences.getInt(NSPreference.FONT_SCALE, NSConstants.DEFAULT_FONT_SCALE);
            if(i == 0)
                i = 100;
            summary = i + "%";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOCATION_TIMEOUT.equals(key))
        {
            preference = findPreference(NSPreference.LOCATION_TIMEOUT);
            value = newValue != null && StrUtil.isNotBlank(newValue.toString()) ? newValue.toString() : sharedPreferences.getString(NSPreference.LOCATION_TIMEOUT, "" + NSConstants.DEFAULT_LOCATION_TIMEOUT);
            i = NSStr.parseInt_s(value, 0);
            summary = i > 0 ? i + "毫秒" : "不超时";
            preference.setSummary(summary);
        }

        if(key == null || NSPreference.LOGCAT_CONSOLE_OUTPUT.equals(key))
        {
            preference = findPreference(NSPreference.LOGCAT_CONSOLE_OUTPUT);
            b = newValue != null ? (Boolean)newValue : sharedPreferences.getBoolean(NSPreference.LOGCAT_CONSOLE_OUTPUT, NSConstants.DEFAULT_LOGCAT_CONSOLE_OUTPUT);
            summary = b ? "控制台输出到logcat" : "不输出控制台";
            preference.setSummary(summary);
        }

        findPreference(NSPreference.VERSION).setSummary(NSContextUtils.GetAppVersion(getContext()));
    }
}

